@@ -1,23 +1,17 @@
-"""
-    sphinx.util.math
-    ~~~~~~~~~~~~~~~~
+"""Utility functions for math."""
 
-    Utility functions for math.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
 from docutils import nodes
 
-from sphinx.builders.html import HTMLTranslator
+from sphinx.builders.html import HTML5Translator
 
 
-def get_node_equation_number(writer: HTMLTranslator, node: nodes.math_block) -> str:
+def get_node_equation_number(writer: HTML5Translator, node: nodes.math_block) -> str:
     if writer.builder.config.math_numfig and writer.builder.config.numfig:
         figtype = 'displaymath'
         if writer.builder.name == 'singlehtml':
-            key = "%s/%s" % (writer.docnames[-1], figtype)
+            key = f"{writer.docnames[-1]}/{figtype}"
         else:
             key = figtype
 
@@ -28,7 +22,7 @@ def get_node_equation_number(writer: HTMLTranslator, node: nodes.math_block) ->
         return node['number']
 
 
-def wrap_displaymath(text: str, label: str, numbering: bool) -> str:
+def wrap_displaymath(text: str, label: str | None, numbering: bool) -> str:
     def is_equation(part: str) -> str:
         return part.strip()
 
@@ -60,4 +54,5 @@ def wrap_displaymath(text: str, label: str, numbering: bool) -> str:
         for part in parts:
             equations.append('%s\\\\\n' % part.strip())
 
-    return '%s\n%s%s' % (begin, ''.join(equations), end)
+    concatenated_equations = ''.join(equations)
+    return f'{begin}\n{concatenated_equations}{end}'
